#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
"""
    Module pyant
    ant by python
"""
import os, sys, stat
import os.path
import shutil, filecmp
import glob
import hashlib
import pyant
from mytools.util import *

class ChdirTask(pyant.Task):
    """
    Change the current directory
    
    name of element : **chdir**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    dir                  the directory for current directory           Yes
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['dir']
    ATTR_DEFAULT = {}
    ATTR_POSSIBLE_VALUE = {} 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _run(self):
        d = os.path.join(self.project.basedir, self._dir)
        os.chdir(d)
        self.log('INFO','Change current directory: %s' % os.path.abspath(d))     

pyant.ANTCLASS['chdir'] = ChdirTask

class Mkdir(pyant.Task):
    """
    Create a directory and parent when necessary
    
    name of element : **mkdir**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    dir                  the directory to create                       Yes
    ifexist              test if exist before create, default true
    ==================== ============================================= ========
    """
    
    ATTR_REQUIRED = ['dir']
    ATTR_DEFAULT = { 'ifexist' : 'true' }
    ATTR_POSSIBLE_VALUE = { 'ifexist' : ['true','false'] } 

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _run(self):
        d = os.path.join(self.project.basedir, self._dir)
        if self._ifexist == 'true':
            if not os.path.isdir(d):
                os.makedirs(d)
                self.log('INFO','Created dir : %s' % os.path.abspath(d))     
            else:
                self.log('INFO','Dir exist : %s ' % os.path.abspath(d))
            return
        os.makedirs(d)
        self.log('INFO','Created dir : %s' % os.path.abspath(d))     

pyant.ANTCLASS['mkdir'] = Mkdir



class Delete(pyant.Task):
    """
    Deletes a single file, a specified directory and all its files and subdirectories
    
    name of element : **delete**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    dir                  the directory to delete                       Yes(1)
    file                 the file to delete                            Yes(1)     
    glob                 pattern to delete                             Yes(1)
    failonerror          stop if failure to delete  default false           
    recursive            from basedir default false
    ==================== ============================================= ========
    
    (1): At least one of the three
    """
    
    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'dir':'', 
                    'file':'', 
                    'glob' : '',
                    'failonerror':'false',
                    'recursive' : 'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false'],
                            'recursive' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._dir + self._file + self._glob) == 0:
            raise AttributeError("%s does not attribute dir or file" % self.__class__.__name__)

    def _run(self):
        try:
            if len(self._file):
                os.remove(os.path.join(d, self._file))
                self.log('INFO','Remove file : %s' % os.path.abspath(os.path.join(d, self._file)))     
                return 
            dirs = [self.project.basedir,]
            if self._recursive == 'true':
                dirs = [root for root, dirnames, filenames in os.walk('.')]
            for d in dirs :
                if len(self._dir):
                    shutil.rmtree(os.path.join(d, self._dir))
                    self.log('INFO','Delete dir : %s' % os.path.abspath(os.path.join(d, self._dir)))
                if len(self._glob):
                    for i in glob.glob(os.path.join(d, self._glob)):
                        if os.path.isdir(i):
                            shutil.rmtree(i)
                            self.log('INFO','Delete dir : %s' % os.path.abspath(i))
                        else:
                            os.remove(i)
                            self.log('INFO','Remove file : %s' % os.path.abspath(i))     
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on delete')
            else:
                raise e



pyant.ANTCLASS['delete'] = Delete

class Chmod(pyant.Task):
    """
    changes permissions of a file or all files inside specified directories
    
    name of element : **chmod**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    dir                  directory which files or directory changed    Yes(1)
    file                 the file or single directory changed          Yes(1)     
    glob                 pattern                                       Yes(1)
    perm                 new permission                                Yes
    failonerror          stop if failure to changed default false           
    ==================== ============================================= ========
    
    (1): At least one of the three
    
    List of permission:

    * S_IRWXU Mask for file owner permissions.
    * S_IRUSR Owner has read permission.
    * S_IWUSR Owner has write permission.
    * S_IXUSR Owner has execute permission.
    * S_IRWXG Mask for group permissions.
    * S_IRGRP Group has read permission.
    * S_IWGRP Group has write permission.
    * S_IXGRP Group has execute permission.
    * S_IRWXO Mask for permissions for others (not in group).
    * S_IROTH Others have read permission.
    * S_IWOTH Others have write permission.
    * S_IXOTH Others have execute permission.
    * S_ENFMT System V file locking enforcement.
    * S_IREAD Unix V7 synonym for S_IRUSR.
    * S_IWRITE Unix V7 synonym for S_IWUSR.
    * S_IEXEC Unix V7 synonym for S_IXUSR.
    * UF_NODUMP Do not dump the file.
    * UF_IMMUTABLE The file may not be changed.
    * UF_APPEND The file may only be appended to.
    * UF_OPAQUE The directory is opaque when viewed through a union stack.
    * UF_NOUNLINK The file may not be renamed or deleted.
    * SF_ARCHIVED The file may be archived.
    * SF_IMMUTABLE The file may not be changed.
    * SF_APPEND The file may only be appended to.
    * SF_NOUNLINK The file may not be renamed or deleted.
    * SF_SNAPSHOT The file is a snapshot file.
    
    """
    PERM = {
        'S_IRWXU' : stat.S_IRWXU,
        'S_IRUSR' : stat.S_IRUSR,
        'S_IWUSR' : stat.S_IWUSR,
        'S_IXUSR' : stat.S_IXUSR,
        'S_IRWXG' : stat.S_IRWXG,
        'S_IRGRP' : stat.S_IRGRP,
        'S_IWGRP' : stat.S_IWGRP,
        'S_IXGRP' : stat.S_IXGRP,
        'S_IRWXO' : stat.S_IRWXO,
        'S_IROTH' : stat.S_IROTH,
        'S_IWOTH' : stat.S_IWOTH,
        'S_IXOTH' : stat.S_IXOTH,
        'S_ENFMT' : stat.S_ENFMT,
        'S_IREAD' : stat.S_IREAD,
        'S_IWRITE' : stat.S_IWRITE,
        'S_IEXEC' : stat.S_IEXEC,
        'UF_NODUMP' : stat.UF_NODUMP,
        'UF_IMMUTABLE' : stat.UF_IMMUTABLE,
        'UF_APPEND' : stat.UF_APPEND,
        'UF_OPAQUE' : stat.UF_OPAQUE,
        'UF_NOUNLINK' : stat.UF_NOUNLINK,
        'SF_ARCHIVED' : stat.SF_ARCHIVED,
        'SF_IMMUTABLE' : stat.SF_IMMUTABLE,
        'SF_APPEND' : stat.SF_APPEND,
        'SF_NOUNLINK' : stat.SF_NOUNLINK,
        'SF_SNAPSHOT' : stat.SF_SNAPSHOT
    } 
    ATTR_REQUIRED = ['perm']
    ATTR_DEFAULT = {'dir':'', 
                    'file':'', 
                    'glob' : '',
                    'failonerror':'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false'],
                            'perm' : PERM.keys()}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._dir + self._file + self._glob) == 0:
            raise AttributeError("%s does not attribute dir or file" % self.__class__.__name__)

    def _run(self):
        try:
            if len(self._file):
                os.chmod(os.path.join(self.project.basedir, self._file), self.PERM[self._perm])
                self.log('INFO','change permission : %s' % os.path.abspath(os.path.join(self.project.basedir, self._file)))     
                return
            dirs = [self.project.basedir,]
            if self._recursive == 'true':
                dirs = [root for root, dirnames, filenames in os.walk('.')]
            for d in dirs :
                if len(self._dir):
                    for i in glob.glob(os.path.join(d, self._dir)):
                        os.chmod(i)
                        self.log('INFO','change permission : %s' % os.path.abspath(os.path.join(d, i)))  
                if len(self._glob):
                    for i in glob.glob(os.path.join(d, self._glob)):
                        os.chmod(i)
                        self.log('INFO','change permission : %s' % os.path.abspath(os.path.join(d, i)))  
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on change permission')
            else:
                raise e

pyant.ANTCLASS['chmod'] = Chmod

class Chown(pyant.Task):
    """
    changes the owner of a file or all files inside specified directories
    
    name of element : **chown**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    dir                  directory which files or directory changed    Yes(1)
    file                 the file or single directory changed          Yes(1)     
    glob                 pattern                                       Yes(1)
    owner                new owner (uid,gid)                           Yes
    failonerror          stop if failure to changed  default false           
    ==================== ============================================= ========
    
    (1): At least one of the three
    """

    ATTR_REQUIRED = ['owner']
    ATTR_DEFAULT = {'dir':'', 
                    'file':'', 
                    'glob' : '',
                    'failonerror':'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._dir + self._file + self._glob) == 0:
            raise AttributeError("%s does not attribute dir or file" % self.__class__.__name__)
        if len(self._owner.split(',')) != 2:
            raise AttributeError("%s does not attribute owner valid uid,gid" % self.__class__.__name__)


    def _run(self):
        try:
            if len(self._file):
                os.chown(os.path.join(self.project.basedir, self._file), self._owner.split(',')[0], self._owner.split(',')[1])
                self.log('INFO','change owner : %s' % os.path.abspath(os.path.join(self.project.basedir, self._file)))     
                return
            dirs = [self.project.basedir,]
            if self._recursive == 'true':
                dirs = [root for root, dirnames, filenames in os.walk('.')]
            for d in dirs :
                if len(self._dir):
                    for i in glob.glob(os.path.join(d, self._dir)):
                        os.chown(i, self._owner.split(',')[0], self._owner.split(',')[1])
                        self.log('INFO','change owner : %s' % os.path.abspath(os.path.join(d, i)))  
                    return
                if len(self._glob):
                    for i in glob.glob(os.path.join(d, self._glob)):
                        os.chown(i, self._owner.split(',')[0], self._owner.split(',')[1])
                        self.log('INFO','change owner : %s' % os.path.abspath(os.path.join(d, i)))  
                    return
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on change owner')
            else:
                raise e

pyant.ANTCLASS['chown'] = Chown


class Copy(pyant.Task):
    """
    copy file or directory in other directory
    
    name of element : **copy**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    file                 the file or single directory to copy          Yes(1)     
    glob                 pattern                                       Yes(1)
    todir                the directory to copy to                      Yes
    failonerror          stop if failure to move  default false           
    ==================== ============================================= ========
    
    (1): At least one of the two
    """

    ATTR_REQUIRED = ['todir']
    ATTR_DEFAULT = {'file':'', 
                    'glob' : '',
                    'failonerror':'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._file + self._glob) == 0:
            raise AttributeError("%s does not attribute dir or file" % self.__class__.__name__)

    def _run(self):
        try:
           action = shutil.copy2
           dest = os.path.join(self.project.basedir, self._todir) 
           if len(self._file):
                action(os.path.join(self.project.basedir, self._file), dest)  
                self.log('INFO','copy %s in %s' % (os.path.join(self.project.basedir, self._file),os.path.join(self.project.basedir, self._todir)))  
                return
           for i in glob.glob(os.path.join(self.project.basedir, self._glob)):
                action(i, dest)  
                self.log('INFO','copy %s in %s' % (i, os.path.join(self.project.basedir, self._todir)))  
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on change owner')
            else:
                raise e

pyant.ANTCLASS['copy'] = Copy

class Move(pyant.Task):
    """
    move file or directory in other directory
    
    name of element : **move**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    file                 the file or single directory to copy          Yes(1)     
    glob                 pattern                                       Yes(1)
    todir                the directory to copy to                      Yes(2)
    tofile               rename a file                                 Yes(2)
    failonerror          stop if failure to move  default false           
    ==================== ============================================= ========
    
    (1): At least one of the two
    
    (2): At least one of the two
    """

    ATTR_REQUIRED = []
    ATTR_DEFAULT = {'file':'', 
                    'glob' : '',
                    'todir' : '',
                    'tofile' : '',
                    'failonerror':'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)

    def _checkadd(self):
        if len(self._file + self._glob) == 0:
            raise AttributeError("%s does not attribute glob or file" % self.__class__.__name__)
        if len(self._tofile + self._todir) == 0:
            raise AttributeError("%s does not attribute todir or tofile" % self.__class__.__name__)

    def _run(self):
        try:
            action = shutil.move
            dest = os.path.join(self.project.basedir, self._todir) 
            if len(self._tofile):
                action = os.rename
                dest = os.path.join(self.project.basedir, self._tofile) 
            if len(self._file):
                action(os.path.join(self.project.basedir, self._file), dest)  
                self.log('INFO','copy %s in %s' % (os.path.join(self.project.basedir, self._file),os.path.join(self.project.basedir, self._todir)))  
                return
            for i in glob.glob(os.path.join(self.project.basedir, self._glob)):
                action(i, dest)  
                self.log('INFO','copy %s in %s' % (i, os.path.join(self.project.basedir, self._todir)))  
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on change owner')
            else:
                raise e

pyant.ANTCLASS['move'] = Move


class Sync(pyant.Task):
    """
    synchronise src and dest
    
    name of element : **sync**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    src                  directory source                              Yes
    dest                 directory destination                         Yes      
    ignore               files separated by ;                                       
    strict               del file in dest if no in src default true
    failonerror          stop if failure to delete  default false           
    ==================== ============================================= ========
    """

    ATTR_REQUIRED = ['src', 'dest']
    ATTR_DEFAULT = {'ignore':'', 
                    'failonerror':'false',
                    'strict' : 'true'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false'],
                            'strict' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)


    def get_cmp_paths(self, dir_cmp, filenames):
        return ((os.path.join(dir_cmp.left, f), os.path.join(dir_cmp.right, f)) for f in filenames)
     
    def sync(self, dir_cmp):
        if self._strict == 'true':
            for f_left, f_right in self.get_cmp_paths(dir_cmp, dir_cmp.right_only):
                if os.path.isfile(f_right):
                    os.remove(f_right)
                else:
                    shutil.rmtree(f_right)
                self.log('DEBUG','delete : %s' %  f_right)
        for f_left, f_right in self.get_cmp_paths(dir_cmp, dir_cmp.left_only+dir_cmp.diff_files):
            if os.path.isfile(f_left):
                shutil.copy2(f_left, f_right)
            else:
                shutil.copytree(f_left, f_right)
            self.log('DEBUG','copy %s' % f_left)  
        for sub_cmp_dir in dir_cmp.subdirs.values():
            self.sync(sub_cmp_dir)

    def _run(self):
        try:
            src = os.path.abspath(os.path.join(self.project.basedir, self._src)) 
            dst = os.path.abspath(os.path.join(self.project.basedir, self._dest)) 
            dir_cmp = filecmp.dircmp(src, dst, ignore=self._ignore.split(';'))
            self.sync(dir_cmp)
            self.log('INFO','synchronize %s with %s' % (src, dst))     
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on sync')
            else:
                raise e

pyant.ANTCLASS['sync'] = Sync

class CheckSum(pyant.Task):
    """
    set property with checksum of file
    
    name of element : **checksum**

    ==================== ============================================= ========
    Attribute            Information                                   Required
    ==================== ============================================= ========
    file                 the file for checksum                         Yes
    property             property with value checksum                  Yes  
    failonerror          stop if failure to move  default false           
    ==================== ============================================= ========
    """

    ATTR_REQUIRED = ['file','property']
    ATTR_DEFAULT = {'failonerror':'false'}
    ATTR_POSSIBLE_VALUE = {'failonerror' : ['true', 'false']}

    def __init__(self, **kwargs):
        pyant.Task.__init__(self, **kwargs)


    def _run(self):
        try:
           self.parent.append_obj(pyant.Property(name=self._property, value= hashlib.md5(os.path.join(self.project.basedir, self._file)).hexdigest()))
        except Exception as e:
            if self._failonerror == 'false':
                self.log('DEBUG','error on checksum of %s' % self._file)
            else:
                raise e

pyant.ANTCLASS['checksum'] = CheckSum











